@@ -22,24 +22,25 @@
  * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
  * Use is subject to license terms.
  */
+/*
+ * This file contains modifications Copyright 2006-2007 J. Schilling
+ *
+ * @(#)sccs.c	1.7 07/01/23 J. Schilling
+ */
+#if defined(sun) || defined(__GNUC__)
+
+#ident "@(#)sccs.c 1.7 07/01/23 J. Schilling"
+#endif
 /*
  * @(#)sccs.c 1.85 06/12/12
  */
 #ifndef lint
 static char sccsid[] = "@(#)sccs.c 1.2 2/27/90";
 #endif
-# include	<schily/mconfig.h>
-# include	<i18n.h>
 # include	<defines.h>
-#ifdef __STDC__
-# include	<stdlib.h>
-/*# include	<dirent.h>*/
+# include	<i18n.h>
 # include	<schily/dirent.h>
-# else 
-/*# include	<dirent.h>*/
-# include	<schily/dirent.h>
-#endif
-# include	<errno.h>
+# include	<schily/errno.h>
 # include	<signal.h>
 #ifdef __STDC__
 #define EX_OK 0
@@ -55,10 +56,13 @@ extern char *getlogin();
 extern char *getenv();
 #endif
 # include	<schily/varargs.h>
-# include	<sys/wait.h>
+# include	<schily/wait.h>
 # include	<pwd.h>
 
-/*static void get_list_files();*/
+#ifdef	NEED_O_BINARY
+#include <io.h>			/* for setmode() prototype */
+#endif
+
 static  char **diffs_np, **diffs_ap;
 
 int	Domrs;
@@ -215,36 +219,12 @@ char    Null[1];
 
 /****************  End of Configuration Information  ****************/
 
-/*typedef char	bool;*/
 typedef int	bool;
 # define TRUE	1
 # define FALSE	0
 
 # define bitset(bit, word)	((bool) ((bit) & (word)))
 
-#ifdef	RRR
-static struct sccsprog *lookup();
-static struct p_file *getpfent();
-static int  checkpfent();
-static void unedit();
-static char *makefile();
-static char *makegfile();
-static char *tail();
-static bool safepath();
-static bool isdir();
-static bool isfile();
-static char *nextfield();
-static void diffs();
-static int  usrerr	__PR((const char *f, ...));
-static void syserr	__PR((const char *f, ...));
-static int  command();
-static int  callprog();
-static int  clean();
-static bool isbranch();
-static void putpfent();
-static char* getNsid();
-#endif
-
 struct sccsprog
 {
 	char	*sccsname;	/* name of SCCS routine */
@@ -409,10 +389,6 @@ struct  stat 		Statbuf;
 extern char	*sys_siglist[];
 #endif
 
-/*static char *gstrcat();*/
-/*static char *gstrncat();*/
-/*static char *gstrcpy();*/
-/*static void gstrbotch();*/
 extern	int Fcnt;
 
 static int create_macro  = 0;	/* 1 if "sccs create ..."  command is running. */
@@ -444,8 +420,13 @@ main(argc, argv)
  	/* 
  	 * Set directory to search for general l10n SCCS messages.
  	 */
+#ifdef	PROTOTYPES
+	bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
+	   NOGETTEXT(INS_BASE "/ccs/lib/locale/"));
+#else
  	bindtextdomain(NOGETTEXT("SUNW_SPRO_SCCS"),
  	   NOGETTEXT("/usr/ccs/lib/locale/"));
+#endif
  	
  	textdomain(NOGETTEXT("SUNW_SPRO_SCCS"));
 
@@ -635,13 +616,12 @@ getNsid(file, user)
 		return(NULL);
 	strcpy(Nsid, "");
 	zero((char *)&goodpf, sizeof(goodpf));
-	in = xfopen(cp, O_RDONLY);
+	in = xfopen(cp, O_RDONLY|O_BINARY);
 	while (fgets(line, sizeof(line), in) != NULL) {
 		pf_ab(line, &pf, 1);
 		if (equal(pf.pf_user, user) || getuid()==0) {
 			if (++cnt) {
 				fclose(in);
-				in = NULL;
 				return (r_option_value);
 			}
 			goodpf = pf;
@@ -649,7 +629,6 @@ getNsid(file, user)
 		}
 	}
 	fclose(in);
-	in = NULL;
 	if (!goodpf.pf_user[0])
 		return (NULL);
 	if(!goodpf.pf_nsid.s_br) {
@@ -859,9 +838,11 @@ command(argv, forkflag, arg0)
 			       break;
 			    dir = readdir(dirf);
 			    dir = readdir(dirf);
-			    while(dir = readdir(dirf)) { 
+			    while ((dir = readdir(dirf)) != NULL) { 
+#ifdef	HAVE_DIRENT_D_INO
 			       if(dir->d_ino == 0)
 				  continue;
+#endif
 			       str = malloc(BUFSIZ);
 			       if(str == NULL) {
 			          perror(gettext("Sccs: no mem"));
@@ -1158,7 +1139,7 @@ command(argv, forkflag, arg0)
 	  case PROG:		/* call an sccs prog */
 	  	if (create_macro == 1 && !strcmp(cmd->sccsname, "get")) {
 	  		char Gname[FILESIZE];
-	  		char *  gf;
+	  		char *  gfp;
 	  		int     ind, ind1 = 0;
 	  		
 			for (ind = 0; ap[ind] != NULL; ind++) {
@@ -1175,14 +1156,14 @@ command(argv, forkflag, arg0)
 					ap_for_get[ind1] = ap[ind1];
 				}
 			}
-		        gf = auxf(ap[ind], 'g');
+		        gfp = auxf(ap[ind], 'g');
 		        strcpy(Gname, SccsPath);
 		        strcat(Gname, "/s.");
-		        strcat(Gname, gf);
+		        strcat(Gname, gfp);
 		        len = strlen(ap[ind]) - strlen(Gname);
 		        strcpy(Gname, "-G");
 		        strncat(Gname, ap[ind], len);
-		        strcat(Gname, gf);
+		        strcat(Gname, gfp);
 			ap_for_get[size_ap_for_get - 3] = Gname;
 		        ap_for_get[size_ap_for_get - 2] = ap[ind];
 			ap_for_get[size_ap_for_get - 1] = NULL;
@@ -2068,12 +2049,7 @@ clean(mode, argv)
 	int mode;
 	char **argv;
 {
-#ifdef __STDC__
 	struct dirent *dir;
-#else
-/*	struct direct *dir;*/
-	struct dirent *dir;
-#endif
 	char buf[MAXPATHLEN];
 	char namefile[MAXPATHLEN];
 	char *bufend;
@@ -2161,7 +2137,7 @@ clean(mode, argv)
 	*/
 
 	gotedit = FALSE;
-	while (dir = readdir(dirfd)) {
+	while ((dir = readdir(dirfd)) != NULL) {
 		if (strncmp(dir->d_name, "s.", 2) != 0) {
 			continue;
 		} else {
@@ -2182,7 +2158,7 @@ clean(mode, argv)
 		**		entry.
 		*/
 
-		pfp = fopen(buf, "r");
+		pfp = fopen(buf, "rb");
 		gotpfent = FALSE;
 		edited = FALSE;
 		if (pfp != NULL)
@@ -2210,7 +2186,6 @@ clean(mode, argv)
 				}
 			}
 			fclose(pfp);
-			pfp = NULL;
 		}
 		
 		/* the s. file exists and no p. file exists -- unlink the g-file */
@@ -2351,7 +2326,7 @@ unedit(fn)
 	/* turn "s." into "p." & try to open it */
 	*++q = 'p';
 
-	pfp = fopen(pfn, "r");
+	pfp = fopen(pfn, "rb");
 	if (pfp == NULL)
 	{
 		printf(gettext("%12s: not being edited\n"),
@@ -2362,16 +2337,19 @@ unedit(fn)
 	/* create temp file for editing p-file */
 	strcpy(tfn, template);
 #ifdef	HAVE_MKSTEMP
-	tfp = fdopen(mkstemp(tfn), "w");
+	tfp = fdopen(mkstemp(tfn), "wb");
 #else
 	mktemp(tfn);
-	tfp = fopen(tfn, "w");
+	tfp = fopen(tfn, "wb");
 #endif
 	if (tfp == NULL)
 	{
 		usrerr(gettext("cannot create \"%s\""), tfn);
 		exit(EX_OSERR);
 	}
+#ifdef	NEED_O_BINARY
+	setmode(fileno(tfp), O_BINARY);
+#endif
 
 	/* figure out who I am */
 	myname = logname();
@@ -2389,9 +2367,7 @@ unedit(fn)
 			if (delete > 1) {
 				printf(gettext("%s: more than one delta of a file is checked out. Use 'sccs unget -r<SID>' instead.\n"), gfile);
 				fclose(tfp);
-				tfp = NULL;
 				fclose(pfp);
-				pfp = NULL;
 				unlink(tfn);
 				return;
 			}
@@ -2428,9 +2404,7 @@ unedit(fn)
 	bad:
 				printf(gettext("%12s: can't remove\n"), gfile);
 				fclose(tfp);
-				tfp = NULL;
 				fclose(pfp);
-				pfp = NULL;
 				unlink(tfn);
 				return;
 			}
@@ -2439,12 +2413,12 @@ unedit(fn)
 	if (others)
 	{
 		/* copy it back (perhaps it should be linked?) */
-		if (freopen(tfn, "r", tfp) == NULL)
+		if (freopen(tfn, "rb", tfp) == NULL)
 		{
 			syserr(gettext("cannot reopen \"%s\""), tfn);
 			exit(EX_OSERR);
 		}
-		if (freopen(pfn, "w", pfp) == NULL)
+		if (freopen(pfn, "wb", pfp) == NULL)
 		{
 			usrerr(gettext("cannot create \"%s\""), pfn);
 			return;
@@ -2465,9 +2439,7 @@ unedit(fn)
 		}
 	}
 	fclose(tfp);
-	tfp = NULL;
 	fclose(pfp);
-	pfp = NULL;
 	unlink(tfn);
 
 	/* actually remove the g-file */
@@ -2790,7 +2762,12 @@ char *file;
 		getcmd = NOGETTEXT("get:Grcixt -s");
 	}
 	strcpy(buf1, template);
+#ifdef	HAVE_MKSTEMP
+	close(mkstemp(buf1));	/* Still a bit safer than mktemp() */
+	chmod(buf1, S_IRUSR);	/* get will not overwrite if writable */
+#else
 	mktemp(buf1);
+#endif
 	tmp_file = buf1;
 	strcpy(buf2, NOGETTEXT("-G"));
 	strcat(buf2, buf1);
